[前][次][番号順一覧][スレッド一覧]

ruby-changes:7877

From: akr <ko1@a...>
Date: Wed, 17 Sep 2008 21:52:33 +0900 (JST)
Subject: [ruby-changes:7877] Ruby:r19398 (trunk): * string.c (rb_str_casecmp): don't use rb_enc_codepoint.

akr	2008-09-17 21:50:52 +0900 (Wed, 17 Sep 2008)

  New Revision: 19398

  http://svn.ruby-lang.org/cgi-bin/viewvc.cgi?view=rev&revision=19398

  Log:
    * string.c (rb_str_casecmp): don't use rb_enc_codepoint.

  Modified files:
    trunk/ChangeLog
    trunk/include/ruby/encoding.h
    trunk/string.c
    trunk/test/ruby/enc/test_utf16.rb

Index: ChangeLog
===================================================================
--- ChangeLog	(revision 19397)
+++ ChangeLog	(revision 19398)
@@ -1,3 +1,7 @@
+Wed Sep 17 21:50:14 2008  Tanaka Akira  <akr@f...>
+
+	* string.c (rb_str_casecmp): don't use rb_enc_codepoint.
+
 Wed Sep 17 19:55:33 2008  Tadayoshi Funaba  <tadf@d...>
 
 	* complex.c (nucomp_s_convert): accepts complex
Index: string.c
===================================================================
--- string.c	(revision 19397)
+++ string.c	(revision 19398)
@@ -1998,7 +1998,6 @@
 static VALUE
 rb_str_casecmp(VALUE str1, VALUE str2)
 {
-    long len;
     rb_encoding *enc;
     char *p1, *p1end, *p2, *p2end;
 
@@ -2013,8 +2012,8 @@
     if (single_byte_optimizable(str1) && single_byte_optimizable(str2)) {
 	while (p1 < p1end && p2 < p2end) {
 	    if (*p1 != *p2) {
-		unsigned int c1 = rb_enc_toupper(*p1 & 0xff, enc);
-		unsigned int c2 = rb_enc_toupper(*p2 & 0xff, enc);
+		unsigned int c1 = TOUPPER(*p1 & 0xff);
+		unsigned int c2 = TOUPPER(*p2 & 0xff);
 		if (c1 > c2) return INT2FIX(1);
 		if (c1 < c2) return INT2FIX(-1);
 	    }
@@ -2024,18 +2023,42 @@
     }
     else {
 	while (p1 < p1end && p2 < p2end) {
-	    unsigned int c1 = rb_enc_codepoint(p1, p1end, enc);
-	    unsigned int c2 = rb_enc_codepoint(p2, p2end, enc);
+            int l1, c1 = rb_enc_ascget(p1, p1end, &l1, enc);
+            int l2, c2 = rb_enc_ascget(p2, p1end, &l2, enc);
 
-	    if (c1 != c2) {
-		c1 = rb_enc_toupper(c1, enc);
-		c2 = rb_enc_toupper(c2, enc);
-		if (c1 > c2) return INT2FIX(1);
-		if (c1 < c2) return INT2FIX(-1);
-	    }
-	    len = rb_enc_codelen(c1, enc);
-	    p1 += len;
-	    p2 += len;
+            if (0 <= c1) {
+                if (0 <= c2) {
+                    if (c1 != c2) {
+                        c1 = TOUPPER(c1);
+                        c2 = TOUPPER(c2);
+                        if (c1 > c2) return INT2FIX(1);
+                        if (c1 < c2) return INT2FIX(-1);
+                    }
+                }
+                else {
+                    return INT2FIX(-1);
+                }
+            }
+            else {
+                if (0 <= c2) {
+                    return INT2FIX(1);
+                }
+                else {
+                    int l, r;
+                    l1 = rb_enc_mbclen(p1, p1end, enc);
+                    l2 = rb_enc_mbclen(p2, p2end, enc);
+                    l = l1;
+                    if (l2 < l)
+                        l = l2;
+                    r = memcmp(p1, p2, l);
+                    if (r != 0)
+                        return INT2FIX(r < 0 ? -1 : 1);
+                    if (l1 != l2)
+                        return INT2FIX(l1 < l2 ? -1 : 1);
+                }
+            }
+	    p1 += l1;
+	    p2 += l2;
 	}
     }
     if (RSTRING_LEN(str1) == RSTRING_LEN(str2)) return INT2FIX(0);
Index: test/ruby/enc/test_utf16.rb
===================================================================
--- test/ruby/enc/test_utf16.rb	(revision 19397)
+++ test/ruby/enc/test_utf16.rb	(revision 19398)
@@ -368,4 +368,16 @@
     r = Regexp.new(Regexp.escape(s))
     assert(r =~ s, "#{encdump(r)} =~ #{encdump(s)}")
   end
+
+  def test_casecmp
+    assert_equal(0, "\0A".force_encoding("UTF-16BE").casecmp("\0a".force_encoding("UTF-16BE")))
+    assert_not_equal(0, "\0A".force_encoding("UTF-16LE").casecmp("\0a".force_encoding("UTF-16LE")))
+    assert_not_equal(0, "A\0".force_encoding("UTF-16BE").casecmp("a\0".force_encoding("UTF-16BE")))
+    assert_equal(0, "A\0".force_encoding("UTF-16LE").casecmp("a\0".force_encoding("UTF-16LE")))
+
+    ary = ["ab".force_encoding("UTF-16LE"), "ba".force_encoding("UTF-16LE")]
+    e = ary.sort {|x,y| x <=> y }
+    a = ary.sort {|x,y| x.casecmp(y) }
+    assert_equal(e, a)
+  end
 end

Index: include/ruby/encoding.h
===================================================================
--- include/ruby/encoding.h	(revision 19397)
+++ include/ruby/encoding.h	(revision 19398)
@@ -125,10 +125,10 @@
 #define rb_enc_mbc_precise_codepoint(p, e, prec_ret, enc) ONIGENC_MBC_PRECISE_CODEPOINT(enc,(UChar*)(p),(UChar*)(e),(prec_ret))
 
 /* -> codelen>0 or raise exception */
-int rb_enc_codelen(int code, rb_encoding *enc);
+int rb_enc_codelen(int codepoint, rb_encoding *enc);
 
 /* code,ptr,encoding -> write buf */
 #define rb_enc_mbcput(c,buf,enc) ONIGENC_CODE_TO_MBC(enc,c,(UChar*)(buf))
 
 /* start, ptr, end, encoding -> prev_char */
 #define rb_enc_prev_char(s,p,e,enc) (char *)onigenc_get_prev_char_head(enc,(UChar*)(s),(UChar*)(p),(UChar*)(e))
@@ -125,10 +125,10 @@
 
-/* code,ptr,encoding -> write buf */
-#define rb_enc_mbcput(c,buf,enc) ONIGENC_CODE_TO_MBC(enc,c,(UChar*)(buf))
+/* codepoint,ptr,encoding -> write buf */
+#define rb_enc_mbcput(codepoint,buf,enc) ONIGENC_CODE_TO_MBC((enc),(codepoint),(UChar*)(buf))
 
 /* start, ptr, end, encoding -> prev_char */
--
ML: ruby-changes@q...
Info: http://www.atdot.net/~ko1/quickml/

[前][次][番号順一覧][スレッド一覧]